@@ -2,7 +2,7 @@ package Mail::Milter::Authentication::Handler::SpamAssassin;
 use strict;
 use warnings;
 use base 'Mail::Milter::Authentication::Handler';
-use version; our $VERSION = version->declare('v1.0.1');
+use version; our $VERSION = version->declare('v1.1.1');
 
 use English qw{ -no_match_vars };
 use Sys::Syslog qw{:standard :macros};
@@ -24,14 +24,21 @@ sub default_config {
         'default_user'   => 'nobody',
         'sa_host'        => 'localhost',
         'sa_port'        => '783',
+        'hard_reject_at' => 10,
         'remove_headers' => 'yes',
     }
 }
 
+sub register_metrics {
+    return {
+        'spamassassin_total' => 'The number of emails processed for SpamAssassin',
+    };
+}
+
 sub get_user {
     my ( $self ) = @_;
-    my $spf_handler = $self->get_handler('UserDB');
-    my $user = $spf_handler->{'local_user'};
+    my $user_handler = $self->get_handler('UserDB');
+    my $user = $user_handler->{'local_user'};
     return $user if $user;
     my $config = $self->handler_config();
     return $config->{'default_user'};
@@ -48,13 +55,15 @@ sub remove_header {
     push @{ $self->{'remove_headers'}->{ lc $key } }, $value;
     return;
 }
-    
+
 sub envfrom_callback {
     my ($self) = @_;
     $self->{'lines'} = [];
     $self->{'rcpt_to'} = q{};
     delete $self->{'header_index'};
     delete $self->{'remove_headers'};
+    $self->{'metrics_data'} = {};
+    $self->{ 'metrics_data' }->{ 'header_removed' } = 'no';
     return;
 }
 
@@ -68,7 +77,7 @@ sub header_callback {
     my ( $self, $header, $value ) = @_;
     push @{$self->{'lines'}} ,$header . ': ' . $value . "\r\n";
     my $config = $self->handler_config();
-    
+
     return if ( $self->is_trusted_ip_address() );
     return if ( lc $config->{'remove_headers'} eq 'no' );
 
@@ -83,6 +92,7 @@ sub header_callback {
             $self->{'header_index'}->{ lc $header_type } =
             $self->{'header_index'}->{ lc $header_type } + 1;
             $self->remove_header( $header_type, $self->{'header_index'}->{ lc $header_type } );
+            $self->{ 'metrics_data' }->{ 'header_removed' } = 'yes';
             if ( lc $config->{'remove_headers'} ne 'silent' ) {
                 my $forged_header =
                   '(Received ' . $header_type . ' header removed by '
@@ -111,12 +121,6 @@ sub body_callback {
     return;
 }
 
-sub epm_requires {
-    my ($self) = @_;
-    my @requires = qw{ UserDB };
-    return \@requires;
-}
-
 sub eom_callback {
     my ($self) = @_;
 
@@ -136,6 +140,9 @@ sub eom_callback {
 
     if ( ! $sa_client->ping() ) {
         $self->log_error( 'SpamAssassin could not connect to server' );
+        $self->add_auth_header('x-spam=temperror');
+        $self->{ 'metrics_data' }->{ 'result' } = 'servererror';
+        $self->metric_count( 'spamassassin_total', $self->{ 'metrics_data' } );
         return;
     }
 
@@ -187,7 +194,21 @@ sub eom_callback {
     );
 
     $self->add_auth_header($header);
-   
+
+    $self->{ 'metrics_data' }->{ 'result' } = ( $sa_status->{'isspam'} eq 'False' ? 'pass' : 'fail' );
+
+    if ( $sa_status->{'isspam'} eq 'True' ) {
+        if ( $config->{'hard_reject_at'} ) {
+            if ( $sa_status->{'score'} >= $config->{'hard_reject_at'} ) {
+                if ( ( ! $self->is_local_ip_address() ) && ( ! $self->is_trusted_ip_address() ) ) {
+                    $self->reject_mail( '550 5.7.0 SPAM policy violation' );
+                    $self->dbgout( 'SpamAssassinReject', "Policy reject", LOG_INFO );
+                }
+            }
+        }
+    }
+
+    $self->metric_count( 'spamassassin_total', $self->{ 'metrics_data' } );
     return if ( lc $config->{'remove_headers'} eq 'no' );
 
     foreach my $header_type ( qw{ X-Spam-score X-Spam-Status X-Spam-hits } ) {
@@ -204,10 +225,12 @@ sub eom_callback {
 
 sub close_callback {
     my ( $self ) = @_;
+
     delete $self->{'lines'};
     delete $self->{'rcpt_to'};
     delete $self->{'remove_headers'};
     delete $self->{'header_index'};
+    delete $self->{'metrics_data'};
     return;
 }
 
@@ -229,6 +252,7 @@ Check email for spam using SpamAssassin spamd.
             "default_user" : "nobody",
             "sa_host" : "localhost",
             "sa_port" : "783",
+            "hard_reject_at" : "10",
             "remove_headers" : "yes"
         },
 
@@ -242,6 +266,7 @@ Add a block to the handlers section of your config as follows.
             "default_user"   : "nobody",
             "sa_host"        : "localhost",
             "sa_port"        : "783",
+            "hard_reject_at" : "10",
             "remove_headers" : "yes"
         },
 
