@@ -5,7 +5,7 @@ use DB_File;
 use Mail::Milter::Authentication::Handler::UserDB::Hash;
 use Sys::Syslog qw{:standard :macros};
 use base 'Mail::Milter::Authentication::Handler';
-use version; our $VERSION = version->declare('v1.0.1');
+use version; our $VERSION = version->declare('v1.1.1');
 
 my $CHECKED_TIME;
 
@@ -16,6 +16,12 @@ sub default_config {
     };
 }
 
+sub register_metrics {
+    return {
+        'userdb_total' => 'The number of emails processed for UserDB',
+    };
+}
+
 sub setup_callback {
     my ( $self ) = @_;
     delete $self->{'local_user'};
@@ -36,6 +42,10 @@ sub eoh_callback {
     return if ! $config->{'add_header'};
     if ( $self->{'local_user'} ) {
         $self->add_auth_header('x-local-user=pass');
+        $self->metric_count( 'userdb_total', { 'result' => 'pass' } );
+    }
+    else {
+        $self->metric_count( 'userdb_total', { 'result' => 'fail' } );
     }
     return;
 }
